package sun.util.resources;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Set;
import sun.util.locale.provider.JRELocaleProviderAdapter;
import java.util.List;
import java.util.MissingResourceException;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.ResourceBundle;
import java.util.Locale;
import sun.util.locale.provider.LocaleProviderAdapter;

public class LocaleData
{
    private final LocaleProviderAdapter.Type type;
    
    public LocaleData(LocaleProviderAdapter.Type type) {
        this.type = type;
    }
    
    public ResourceBundle getCalendarData(Locale locale) {
        return getBundle(this.type.getUtilResourcesPackage() + ".CalendarData", locale);
    }
    
    public OpenListResourceBundle getCurrencyNames(Locale locale) {
        return (OpenListResourceBundle)getBundle(this.type.getUtilResourcesPackage() + ".CurrencyNames", locale);
    }
    
    public OpenListResourceBundle getLocaleNames(Locale locale) {
        return (OpenListResourceBundle)getBundle(this.type.getUtilResourcesPackage() + ".LocaleNames", locale);
    }
    
    public TimeZoneNamesBundle getTimeZoneNames(Locale locale) {
        return (TimeZoneNamesBundle)getBundle(this.type.getUtilResourcesPackage() + ".TimeZoneNames", locale);
    }
    
    public ResourceBundle getBreakIteratorInfo(Locale locale) {
        return getBundle(this.type.getTextResourcesPackage() + ".BreakIteratorInfo", locale);
    }
    
    public ResourceBundle getCollationData(Locale locale) {
        return getBundle(this.type.getTextResourcesPackage() + ".CollationData", locale);
    }
    
    public ResourceBundle getDateFormatData(Locale locale) {
        return getBundle(this.type.getTextResourcesPackage() + ".FormatData", locale);
    }
    
    public void setSupplementary(ParallelListResourceBundle parallelListResourceBundle) {
        if (!parallelListResourceBundle.areParallelContentsComplete()) {
            this.setSupplementary(this.type.getTextResourcesPackage() + ".JavaTimeSupplementary", parallelListResourceBundle);
        }
    }
    
    private boolean setSupplementary(String s, ParallelListResourceBundle parallelListResourceBundle) {
        ParallelListResourceBundle parallelListResourceBundle2 = (ParallelListResourceBundle)parallelListResourceBundle.getParent();
        boolean setSupplementary = false;
        if (parallelListResourceBundle2 != null) {
            setSupplementary = this.setSupplementary(s, parallelListResourceBundle2);
        }
        OpenListResourceBundle supplementary = getSupplementary(s, parallelListResourceBundle.getLocale());
        parallelListResourceBundle.setParallelContents(supplementary);
        int n = setSupplementary | ((supplementary != null) ? 1 : 0);
        if (!(n == 0)) {
            parallelListResourceBundle.resetKeySet();
        }
        return n;
    }
    
    public ResourceBundle getNumberFormatData(Locale locale) {
        return getBundle(this.type.getTextResourcesPackage() + ".FormatData", locale);
    }
    
    public static ResourceBundle getBundle(final String s, final Locale locale) {
        return (ResourceBundle)AccessController.doPrivileged(new PrivilegedAction<ResourceBundle>() {
            @Override
            public ResourceBundle run() {
                return ResourceBundle.getBundle(s, locale, LocaleData.LocaleDataResourceBundleControl.INSTANCE);
            }
        });
    }
    
    private static OpenListResourceBundle getSupplementary(final String s, final Locale locale) {
        return (OpenListResourceBundle)AccessController.doPrivileged(new PrivilegedAction<OpenListResourceBundle>() {
            @Override
            public OpenListResourceBundle run() {
                OpenListResourceBundle openListResourceBundle = null;
                try {
                    openListResourceBundle = (OpenListResourceBundle)ResourceBundle.getBundle(s, locale, LocaleData.SupplementaryResourceBundleControl.INSTANCE);
                }
                catch (MissingResourceException ex) {}
                return openListResourceBundle;
            }
        });
    }
    
    private static class LocaleDataResourceBundleControl extends ResourceBundle.Control
    {
        private static final LocaleData.LocaleDataResourceBundleControl INSTANCE;
        private static final String DOTCLDR = ".cldr";
        
        @Override
        public List<Locale> getCandidateLocales(String s, Locale locale) {
            List candidateLocales = super.getCandidateLocales(s, locale);
            int lastIndex = s.lastIndexOf(46);
            s = ((lastIndex >= 0) ? s.substring(lastIndex + 1) : s);
            LocaleProviderAdapter.Type type = s.contains(".cldr") ? LocaleProviderAdapter.Type.CLDR : LocaleProviderAdapter.Type.JRE;
            Set<String> languageTagSet = ((JRELocaleProviderAdapter)LocaleProviderAdapter.forType(type)).getLanguageTagSet(s);
            if (!languageTagSet.isEmpty()) {
                Iterator<Object> iterator = candidateLocales.iterator();
                while (iterator.hasNext()) {
                    if (LocaleProviderAdapter.isSupportedLocale((Locale)iterator.next(), type, languageTagSet)) {
                        continue;
                    }
                    iterator.remove();
                }
            }
            if (locale.getLanguage() != "en" && type == LocaleProviderAdapter.Type.CLDR && s.equals("TimeZoneNames")) {
                candidateLocales.add(candidateLocales.size() - 1, Locale.ENGLISH);
                return candidateLocales;
            }
            return candidateLocales;
        }
        
        @Override
        public Locale getFallbackLocale(String s, Locale locale) {
            if (s == null || locale == null) {
                throw new NullPointerException();
            }
            return null;
        }
        
        @Override
        public String toBundleName(String string, Locale locale) {
            String language = locale.getLanguage();
            if (language.length() > 0 && (string.startsWith(LocaleProviderAdapter.Type.JRE.getUtilResourcesPackage()) || string.startsWith(LocaleProviderAdapter.Type.JRE.getTextResourcesPackage()))) {
                assert LocaleProviderAdapter.Type.JRE.getUtilResourcesPackage().length() == LocaleProviderAdapter.Type.JRE.getTextResourcesPackage().length();
                int length = LocaleProviderAdapter.Type.JRE.getUtilResourcesPackage().length();
                if (string.indexOf(".cldr", length) > 0) {
                    length += ".cldr".length();
                }
                string = string.substring(0, length + 1) + language + string.substring(length);
            }
            return super.toBundleName(string, locale);
        }
        
        static {
            INSTANCE = new LocaleData.LocaleDataResourceBundleControl();
        }
    }
    
    private static class SupplementaryResourceBundleControl extends LocaleData.LocaleDataResourceBundleControl
    {
        private static final LocaleData.SupplementaryResourceBundleControl INSTANCE;
        
        @Override
        public List<Locale> getCandidateLocales(String s, Locale locale) {
            return Arrays.asList(new Locale[] { locale });
        }
        
        @Override
        public long getTimeToLive(String s, Locale locale) {
            assert !(!s.contains("JavaTimeSupplementary"));
            return -1L;
        }
        
        static {
            INSTANCE = new LocaleData.SupplementaryResourceBundleControl();
        }
    }
}
